@@ -3,6 +3,7 @@
 # Licensed under the 2-clause BSD License
 
 """Class for reading and writing uvfits files."""
+import os
 import warnings
 
 import numpy as np
@@ -40,7 +41,7 @@ class UVFITS(UVData):
     ]
 
     def _get_parameter_data(
-        self, vis_hdu, run_check_acceptability, background_lsts=True
+        self, vis_hdu, run_check_acceptability, background_lsts=True,
     ):
         """
         Read just the random parameters portion of the uvfits file ("metadata").
@@ -115,14 +116,13 @@ class UVFITS(UVData):
                 bl_input_array
             )
 
-        # check for multi source files
+        # check for multi source files. NOW SUPPORTED, W00T!
         if "SOURCE" in vis_hdu.data.parnames:
+            # Preserve the source info just in case the AIPS SU table is missing, and
+            # we need to revert things back.
+            self._set_multi_phase_center(preserve_phase_center_info=True)
             source = vis_hdu.data.par("SOURCE")
-            if len(set(source)) > 1:
-                raise ValueError(
-                    "This file has multiple sources. Only single "
-                    "source observations are supported."
-                )
+            self.phase_center_id_array = source.astype(int)
 
         # get self.baseline_array using our convention
         self.baseline_array = self.antnums_to_baseline(
@@ -162,7 +162,7 @@ class UVFITS(UVData):
                 )
             else:
                 raise ValueError(
-                    "integration time not specified and only " "one time present"
+                    "integration time not specified and only one time present"
                 )
 
         if proc is not None:
@@ -179,6 +179,8 @@ class UVFITS(UVData):
         freq_chans,
         times,
         time_range,
+        lsts,
+        lst_range,
         polarizations,
         blt_inds,
         read_metadata,
@@ -187,6 +189,8 @@ class UVFITS(UVData):
         check_extra,
         run_check_acceptability,
         strict_uvw_antpos_check,
+        fix_old_proj,
+        fix_use_ant_pos,
     ):
         """
         Read just the visibility and flag data of the uvfits file.
@@ -203,6 +207,8 @@ class UVFITS(UVData):
             freq_chans,
             times,
             time_range,
+            lsts,
+            lst_range,
             polarizations,
             blt_inds,
         )
@@ -309,6 +315,9 @@ class UVFITS(UVData):
         self.flag_array = raw_data_array[:, :, :, :, 2] <= 0
         self.nsample_array = np.abs(raw_data_array[:, :, :, :, 2])
 
+        if fix_old_proj:
+            self.fix_phase(use_ant_pos=fix_use_ant_pos)
+
         # check if object has all required UVParameters set
         if run_check:
             self.check(
@@ -328,6 +337,8 @@ class UVFITS(UVData):
         freq_chans=None,
         times=None,
         time_range=None,
+        lsts=None,
+        lst_range=None,
         polarizations=None,
         blt_inds=None,
         keep_all_metadata=True,
@@ -337,6 +348,8 @@ class UVFITS(UVData):
         check_extra=True,
         run_check_acceptability=True,
         strict_uvw_antpos_check=False,
+        fix_old_proj=False,
+        fix_use_ant_pos=True,
     ):
         """
         Read in header, metadata and data from a uvfits file.
@@ -391,6 +404,17 @@ class UVFITS(UVData):
             The time range in Julian Date to keep in the object, must be
             length 2. Some of the times in the object should fall between the
             first and last elements. Cannot be used with `times`.
+        lsts : array_like of float, optional
+            The local sidereal times (LSTs) to keep in the object, each value
+            passed here should exist in the lst_array. Cannot be used with
+            `times`, `time_range`, or `lst_range`.
+        lst_range : array_like of float, optional
+            The local sidereal time (LST) range in radians to keep in the
+            object, must be of length 2. Some of the LSTs in the object should
+            fall between the first and last elements. If the second value is
+            smaller than the first, the LSTs are treated as having phase-wrapped
+            around LST = 2*pi = 0, and the LSTs kept on the object will run from
+            the larger value, through 0, and end at the smaller value.
         polarizations : array_like of int, optional
             The polarizations numbers to include when reading data into the
             object, each value passed here should exist in the polarization_array.
@@ -422,6 +446,15 @@ class UVFITS(UVData):
         strict_uvw_antpos_check : bool
             Option to raise an error rather than a warning if the check that
             uvws match antenna positions does not pass.
+        fix_old_proj : bool
+            Applies a fix to uvw-coordinates and phasing, assuming that the old `phase`
+            method was used prior to writing the data, which had errors of the order of
+            one part in 1e4 - 1e5. See the phasing memo for more details. Default is
+            False.
+        fix_use_ant_pos : bool
+            If setting `fix_old_proj` to True, use the antenna positions to derive the
+            correct uvw-coordinates rather than using the baseline vectors. Default is
+            True.
 
 
         Raises
@@ -432,11 +465,15 @@ class UVFITS(UVData):
             If incompatible select keywords are set (e.g. `ant_str` with other
             antenna selectors, `times` and `time_range`) or select keywords
             exclude all data or if keywords are set to the wrong type.
-            If the data are multi source.
             If the data have multi spw with different channel widths.
             If the metadata are not internally consistent or missing.
 
         """
+        # update filename attribute
+        basename = os.path.basename(filename)
+        self.filename = [basename]
+        self._filename.form = (1,)
+
         with fits.open(filename, memmap=True) as hdu_list:
             vis_hdu = hdu_list[0]  # assumes the visibilities are in the primary hdu
             vis_hdr = vis_hdu.header.copy()
@@ -525,7 +562,31 @@ class UVFITS(UVData):
 
             self.vis_units = vis_hdr.pop("BUNIT", "UNCALIB")
             self.phase_center_epoch = vis_hdr.pop("EPOCH", None)
+
+            # PHSFRAME is not a standard UVFITS keyword, but was used by older
+            # versions of pyuvdata. To ensure backwards compatibility, we look
+            # for it first to determine the coordinate frame for the data
             self.phase_center_frame = vis_hdr.pop("PHSFRAME", None)
+            # If we don't find the special keyword PHSFRAME, try for the more
+            # FITS-standard RADESYS
+            if self.phase_center_frame is None:
+                self.phase_center_frame = vis_hdr.pop("RADESYS", None)
+            # If we still don't find anything, try the two 'special' variant names
+            # for the coordinate frame that seem to have been documented
+            if self.phase_center_frame is None:
+                self.phase_center_frame = vis_hdr.pop("RADESYSA", None)
+            if self.phase_center_frame is None:
+                self.phase_center_frame = vis_hdr.pop("RADESYSa", None)
+
+            # If we _still_ can't find anything, take a guess based on the value
+            # listed in the EPOCH. The behavior listed here is based off of the
+            # AIPS task REGRD (http://www.aips.nrao.edu/cgi-bin/ZXHLP2.PL?REGRD)
+            if self.phase_center_frame is None:
+                if self.phase_center_epoch is None:
+                    self.phase_center_frame = "icrs"
+                else:
+                    frame = "fk4" if (self.phase_center_epoch == 1950.0) else "fk5"
+                    self.phase_center_frame = frame
 
             self.extra_keywords = uvutils._get_fits_extra_keywords(
                 vis_hdr, keywords_to_skip=["DATE-OBS"]
@@ -651,8 +712,77 @@ class UVFITS(UVData):
 
             # Now read in the random parameter info
             self._get_parameter_data(
-                vis_hdu, run_check_acceptability, background_lsts=background_lsts
+                vis_hdu, run_check_acceptability, background_lsts=background_lsts,
             )
+            # If we find the source attribute in the FITS random paramter list,
+            # the multi_phase_center attribute will be set to True, and we should also
+            # expect that there must be an AIPS SU table.
+            if self.multi_phase_center and "AIPS SU" not in hdunames.keys():
+                warnings.warn(
+                    "UVFITS file is missing AIPS SU table, which is required when "
+                    "SOURCE is one of the `random paramters` in the main binary "
+                    "table. Bypassing for now, but note that this file _may_ not "
+                    "work correctly in UVFITS-based programs (e.g., AIPS, CASA)."
+                )
+                name = list(self.phase_center_catalog.keys())[0]
+                self.phase_center_ra = self.phase_center_catalog[name]["cat_lon"]
+                self.phase_center_dec = self.phase_center_catalog[name]["cat_lat"]
+                self.phase_center_frame = self.phase_center_catalog[name]["cat_frame"]
+                self.phase_center_epoch = self.phase_center_catalog[name]["cat_epoch"]
+
+                self.multi_phase_center = False
+                self._phase_center_id_array.required = False
+                self._Nphase.required = False
+                self._phase_center_catalog.required = False
+                self.object_name = name
+                self.Nphase = None
+                self.phase_center_catalog = None
+                self.phase_center_id_array = None
+            elif self.multi_phase_center:
+                su_hdu = hdu_list[hdunames["AIPS SU"]]
+                # We should have as many entries in the AIPS SU header as we have
+                # unique entries in the SOURCES random paramter (checked in the call
+                # to get_parameter_data above)
+                if len(su_hdu.data) != len(np.unique(self.phase_center_id_array)):
+                    raise RuntimeError(
+                        "The UVFITS file has a malformed AIPS SU table - number of "
+                        "sources do not match the number of unique source IDs in the "
+                        "primary data header."
+                    )  # pragma: no cover
+
+                # Reset the catalog, since it has some dummy information stored within
+                # it (that was pulled off the primary table)
+                self._remove_phase_center(list(self.phase_center_catalog.keys())[0])
+
+                # Set up these arrays so we can assign values to them
+                self.phase_center_app_ra = np.zeros(self.Nblts)
+                self.phase_center_app_dec = np.zeros(self.Nblts)
+                self.phase_center_app_pa = np.zeros(self.Nblts)
+
+                # Alright, we are off to the races!
+                for idx in range(len(su_hdu.data)):
+                    # Grab the indv source entry
+                    sou_info = su_hdu.data[idx]
+                    sou_id = sou_info["ID. NO."]
+                    sou_name = sou_info["SOURCE"]
+                    sou_ra = sou_info["RAEPO"] * (np.pi / 180.0)
+                    sou_dec = sou_info["DECEPO"] * (np.pi / 180.0)
+                    sou_epoch = sou_info["EPOCH"]
+                    sou_frame = "fk5"
+
+                    self._add_phase_center(
+                        sou_name,
+                        cat_id=sou_id,
+                        cat_type="sidereal",
+                        cat_lon=sou_ra,
+                        cat_lat=sou_dec,
+                        cat_frame=sou_frame,
+                        cat_epoch=sou_epoch,
+                        info_source="uvfits file",
+                    )
+
+            # Calculate the apparent coordinate values
+            self._set_app_coords_helper()
 
             if not read_data:
                 # don't read in the data. This means the object is a metadata
@@ -670,6 +800,8 @@ class UVFITS(UVData):
                 freq_chans,
                 times,
                 time_range,
+                lsts,
+                lst_range,
                 polarizations,
                 blt_inds,
                 False,
@@ -678,6 +810,8 @@ class UVFITS(UVData):
                 check_extra,
                 run_check_acceptability,
                 strict_uvw_antpos_check,
+                fix_old_proj,
+                fix_use_ant_pos,
             )
 
     def write_uvfits(
@@ -723,7 +857,6 @@ class UVFITS(UVData):
         ValueError
             The `phase_type` of the object is "drift" and the `force_phase`
             keyword is not set.
-            The `phase_type` of the object is "unknown".
             If the frequencies are not evenly spaced or are separated by more
             than their channel width.
             The polarization values are not evenly spaced.
@@ -760,12 +893,7 @@ class UVFITS(UVData):
                     "to zenith of the first timestamp before "
                     "writing a uvfits file."
                 )
-        else:
-            raise ValueError(
-                "The phasing type of the data is unknown. "
-                "Set the phase_type to drift or phased to "
-                "reflect the phasing status of the data"
-            )
+
         if self.flex_spw:
             # If we have a 'flexible' spectral window, we will need to evaluate the
             # frequency axis slightly differently.
@@ -832,6 +960,9 @@ class UVFITS(UVData):
 
         if self.Npols > 1:
             pol_spacing = np.diff(self.polarization_array)
+            pol_indexing = np.argsort(np.abs(self.polarization_array))
+            polarization_array = self.polarization_array[pol_indexing]
+            pol_spacing = np.diff(polarization_array)
             if np.min(pol_spacing) < np.max(pol_spacing):
                 raise ValueError(
                     "The polarization values are not evenly spaced (probably "
@@ -840,6 +971,8 @@ class UVFITS(UVData):
                 )
             pol_spacing = pol_spacing[0]
         else:
+            pol_indexing = np.asarray([0])
+            polarization_array = self.polarization_array
             pol_spacing = 1
 
         for p in self.extra():
@@ -883,6 +1016,8 @@ class UVFITS(UVData):
         weights_array = np.reshape(
             self.nsample_array * np.where(self.flag_array, -1, 1), uvfits_data_shape,
         )
+        data_array = data_array[:, :, :, :, :, pol_indexing, :]
+        weights_array = weights_array[:, :, :, :, :, pol_indexing, :]
 
         uvfits_array_data = np.concatenate(
             [data_array.real, data_array.imag, weights_array], axis=6
@@ -912,7 +1047,7 @@ class UVFITS(UVData):
             "WW      ": uvw_array_sec[:, 2],
             "DATE    ": time_array,
             "BASELINE": baselines_use,
-            # "SOURCE  ": np.ones(time_array.shape, dtype=np.float32),
+            "SOURCE  ": None,
             "FREQSEL ": np.ones_like(self.time_array, dtype=np.float32),
             "ANTENNA1": self.ant_1_array + 1,
             "ANTENNA2": self.ant_2_array + 1,
@@ -920,13 +1055,22 @@ class UVFITS(UVData):
             "INTTIM  ": int_time_array,
         }
 
+        if self.multi_phase_center:
+            id_offset = np.any(
+                [
+                    temp_dict["cat_id"] == 0
+                    for temp_dict in self.phase_center_catalog.values()
+                ]
+            )
+            group_parameter_dict["SOURCE  "] = self.phase_center_id_array + id_offset
+
         pscal_dict = {
             "UU      ": 1.0,
             "VV      ": 1.0,
             "WW      ": 1.0,
             "DATE    ": 1.0,
             "BASELINE": 1.0,
-            # "SOURCE  ": 1.0,
+            "SOURCE  ": 1.0,
             "FREQSEL ": 1.0,
             "ANTENNA1": 1.0,
             "ANTENNA2": 1.0,
@@ -939,7 +1083,7 @@ class UVFITS(UVData):
             "WW      ": 0.0,
             "DATE    ": tzero,
             "BASELINE": 0.0,
-            # "SOURCE  ": 0.0,
+            "SOURCE  ": 0.0,
             "FREQSEL ": 0.0,
             "ANTENNA1": 0.0,
             "ANTENNA2": 0.0,
@@ -968,7 +1112,8 @@ class UVFITS(UVData):
             # Otherwise just use the antenna arrays
             parnames_use.append("BASELINE")
 
-        # TODO Karto: Here's where to add "SOURCE " item, and potentially "FREQSEL "
+        if self.multi_phase_center:
+            parnames_use.append("SOURCE  ")
 
         parnames_use += ["ANTENNA1", "ANTENNA2", "SUBARRAY", "INTTIM  "]
 
@@ -1008,7 +1153,7 @@ class UVFITS(UVData):
         # However, this confusing because it is NOT a true Stokes axis,
         #   it is really the polarization axis.
         hdu.header["CTYPE3  "] = "STOKES  "
-        hdu.header["CRVAL3  "] = float(self.polarization_array[0])
+        hdu.header["CRVAL3  "] = float(polarization_array[0])
         hdu.header["CRPIX3  "] = 1.0
         hdu.header["CDELT3  "] = float(pol_spacing)
 
@@ -1032,15 +1177,23 @@ class UVFITS(UVData):
         hdu.header["BSCALE  "] = 1.0
         hdu.header["BZERO   "] = 0.0
 
-        hdu.header["OBJECT  "] = self.object_name
+        name = "MULTI" if self.multi_phase_center else self.object_name
+        hdu.header["OBJECT  "] = name
         hdu.header["TELESCOP"] = self.telescope_name
         hdu.header["LAT     "] = self.telescope_location_lat_lon_alt_degrees[0]
         hdu.header["LON     "] = self.telescope_location_lat_lon_alt_degrees[1]
         hdu.header["ALT     "] = self.telescope_location_lat_lon_alt[2]
         hdu.header["INSTRUME"] = self.instrument
-        hdu.header["EPOCH   "] = float(self.phase_center_epoch)
+        if self.phase_center_epoch is not None:
+            hdu.header["EPOCH   "] = float(self.phase_center_epoch)
+        # TODO: This is a keyword that should at some point get added for velocity
+        # reference stuff, although for right now pyuvdata doesn't do any sort of
+        # handling of this, so stub this out for now.
+        # hdu.header["SPECSYS "] = "TOPOCENT"
+
         if self.phase_center_frame is not None:
-            hdu.header["PHSFRAME"] = self.phase_center_frame
+            # Previous versions of pyuvdata wrote this header as PHSFRAME
+            hdu.header["RADESYS"] = self.phase_center_frame
 
         if self.x_orientation is not None:
             hdu.header["XORIENT"] = self.x_orientation
@@ -1096,19 +1249,21 @@ class UVFITS(UVData):
             self.antenna_positions, longitude
         )
         col2 = fits.Column(name="STABXYZ", format="3D", array=rot_ecef_positions)
+        # col3 = fits.Column(name="ORBPARAM", format="0D", array=Norb)
         # convert to 1-indexed from 0-indexed indicies
-        col3 = fits.Column(name="NOSTA", format="1J", array=self.antenna_numbers + 1)
-        col4 = fits.Column(name="MNTSTA", format="1J", array=mntsta)
-        col5 = fits.Column(name="STAXOF", format="1E", array=staxof)
-        col6 = fits.Column(name="POLTYA", format="1A", array=poltya)
-        col7 = fits.Column(name="POLAA", format="1E", array=polaa)
-        # col8 = fits.Column(name='POLCALA', format='3E', array=polcala)
-        col9 = fits.Column(name="POLTYB", format="1A", array=poltyb)
-        col10 = fits.Column(name="POLAB", format="1E", array=polab)
-        # col11 = fits.Column(name='POLCALB', format='3E', array=polcalb)
-        # note ORBPARM is technically required, but we didn't put it in
-        col_list = [col1, col2, col3, col4, col5, col6, col7, col9, col10]
-
+        col4 = fits.Column(name="NOSTA", format="1J", array=self.antenna_numbers + 1)
+        col5 = fits.Column(name="MNTSTA", format="1J", array=mntsta)
+        col6 = fits.Column(name="STAXOF", format="1E", array=staxof)
+        col7 = fits.Column(name="POLTYA", format="1A", array=poltya)
+        col8 = fits.Column(name="POLAA", format="1E", array=polaa)
+        # col9 = fits.Column(name='POLCALA', format='0E', array=Npcal, Nspws)
+        col10 = fits.Column(name="POLTYB", format="1A", array=poltyb)
+        col11 = fits.Column(name="POLAB", format="1E", array=polab)
+        # col12 = fits.Column(name='POLCALB', format='0E', array=Npcal, Nspws)
+        col_list = [col1, col2, col4, col5, col6, col7, col8, col10, col11]
+        # The commented out entires are up above to help check for consistency with the
+        # UVFITS format. ORBPARAM, POLCALA, and POLCALB are all technically required,
+        # but are all of zero length. Added here to help with debugging.
         if self.antenna_diameters is not None:
             col12 = fits.Column(
                 name="DIAMETER", format="1E", array=self.antenna_diameters
@@ -1145,7 +1300,8 @@ class UVFITS(UVData):
         ant_hdu.header["ARRNAM"] = self.telescope_name
         ant_hdu.header["NO_IF"] = self.Nspws
         ant_hdu.header["DEGPDY"] = self.earth_omega
-        # ant_hdu.header['IATUTC'] = 35.
+        # This is just a statically defined value
+        ant_hdu.header["IATUTC"] = 37.0
 
         # set mandatory parameters which are not supported by this object
         # (or that we just don't understand)
@@ -1170,6 +1326,13 @@ class UVFITS(UVData):
         # we always output right handed coordinates
         ant_hdu.header["XYZHAND"] = "RIGHT"
 
+        # At some point, we can fill these in more completely using astropy IERS
+        # utilities, since CASA/AIPS doesn't want to be told what the apparent coords
+        # are, but rather wants to calculate them itself.
+        # ant_hdu.header["RDATE"] = '2020-07-24T16:35:39.144087'
+        # ant_hdu.header["POLARX"] = 0.0
+        # ant_hdu.header["POLARY"] = 0.0
+
         fits_tables = [hdu, ant_hdu]
         # If needed, add the FQ table
         if self.Nspws > 1:
@@ -1201,6 +1364,103 @@ class UVFITS(UVData):
             fq_hdu.header["NO_IF"] = self.Nspws
             fits_tables.append(fq_hdu)
 
+        # If needed, add the SU table
+        if self.multi_phase_center:
+            fmt_d = "%iD" % self.Nspws
+            fmt_e = "%iE" % self.Nspws
+            fmt_j = "%iJ" % self.Nspws
+
+            int_zeros = np.zeros(self.Nphase, dtype=int)
+            flt_zeros = np.zeros(self.Nphase, dtype=np.float64)
+            zero_arr = np.zeros((self.Nphase, self.Nspws))
+            sou_ids = np.zeros(self.Nphase)
+            name_arr = np.array(list(self.phase_center_catalog.keys()))
+            cal_code = ["    "] * self.Nphase
+            # These are things we need to flip through on a source-by-source basis
+            ra_arr = np.zeros(self.Nphase, dtype=np.float64)
+            app_ra = np.zeros(self.Nphase, dtype=np.float64)
+            dec_arr = np.zeros(self.Nphase, dtype=np.float64)
+            app_dec = np.zeros(self.Nphase, dtype=np.float64)
+            epo_arr = np.zeros(self.Nphase, dtype=np.float64)
+            pm_ra = np.zeros(self.Nphase, dtype=np.float64)
+            pm_dec = np.zeros(self.Nphase, dtype=np.float64)
+            rest_freq = np.zeros((self.Nphase, self.Nspws), dtype=np.float64)
+            for idx, name in enumerate(name_arr):
+                phase_dict = self.phase_center_catalog[name]
+                # This is a stub for something smarter in the future
+                sou_ids[idx] = self.phase_center_catalog[name]["cat_id"] + id_offset
+                rest_freq[idx][:] = np.mean(self.freq_array)
+                pm_ra[idx] = 0.0
+                pm_dec[idx] = 0.0
+                if phase_dict["cat_type"] == "sidereal":
+                    # So here's the deal -- we need all the objects to be in the same
+                    # coordinate frame, although nothing in phase_center_catalog forces
+                    # objects to share the same frame. So we want to make sure that
+                    # everything lines up with the coordinate frame listed.
+                    ra_arr[idx], dec_arr[idx] = uvutils.transform_sidereal_coords(
+                        phase_dict["cat_lon"],
+                        phase_dict["cat_lat"],
+                        phase_dict["cat_frame"],
+                        "fk5",
+                        in_coord_epoch=phase_dict.get("cat_epoch"),
+                        out_coord_epoch=phase_dict.get("cat_epoch"),
+                        time_array=np.mean(self.time_array),
+                    )
+
+                    epo_arr[idx] = (
+                        phase_dict["cat_epoch"]
+                        if "cat_epoch" in (phase_dict.keys())
+                        else 2000.0
+                    )
+
+                    cat_id = self.phase_center_catalog[name]["cat_id"]
+
+                    app_ra[idx] = np.median(
+                        self.phase_center_app_ra[self.phase_center_id_array == cat_id]
+                    )
+
+                    app_dec[idx] = np.median(
+                        self.phase_center_app_dec[self.phase_center_id_array == cat_id]
+                    )
+
+                ra_arr *= 180.0 / np.pi
+                dec_arr *= 180.0 / np.pi
+                app_ra *= 180.0 / np.pi
+                app_dec *= 180.0 / np.pi
+
+            col_list = [
+                fits.Column(name="ID. NO.", format="1J", array=sou_ids),
+                fits.Column(name="SOURCE", format="20A", array=name_arr),
+                fits.Column(name="QUAL", format="1J", array=int_zeros),
+                fits.Column(name="CALCODE", format="4A", array=cal_code),
+                fits.Column(name="IFLUX", format=fmt_e, unit="JY", array=zero_arr),
+                fits.Column(name="QFLUX", format=fmt_e, unit="JY", array=zero_arr),
+                fits.Column(name="UFLUX", format=fmt_e, unit="JY", array=zero_arr),
+                fits.Column(name="VFLUX", format=fmt_e, unit="JY", array=zero_arr),
+                fits.Column(name="FREQOFF", format=fmt_d, unit="HZ", array=zero_arr),
+                fits.Column(name="BANDWIDTH", format="1D", unit="HZ", array=flt_zeros),
+                fits.Column(name="RAEPO", format="1D", unit="DEGREES", array=ra_arr),
+                fits.Column(name="DECEPO", format="1D", unit="DEGREES", array=dec_arr),
+                fits.Column(name="EPOCH", format="1D", unit="YEARS", array=epo_arr),
+                fits.Column(name="RAAPP", format="1D", unit="DEGREES", array=app_ra),
+                fits.Column(name="DECAPP", format="1D", unit="DEGREES", array=app_dec),
+                fits.Column(name="LSRVEL", format=fmt_d, unit="M/SEC", array=zero_arr),
+                fits.Column(name="RESTFREQ", format=fmt_d, unit="HZ", array=rest_freq),
+                fits.Column(name="PMRA", format="1D", unit="DEG/DAY", array=pm_ra),
+                fits.Column(name="PMDEC", format="1D", unit="DEG/DAY", array=pm_dec),
+            ]
+
+            su_hdu = fits.BinTableHDU.from_columns(fits.ColDefs(col_list))
+            su_hdu.header["EXTNAME"] = "AIPS SU"
+            su_hdu.header["NO_IF"] = self.Nspws
+            su_hdu.header["FREQID"] = 1
+            su_hdu.header["VELDEF"] = "RADIO"
+            # TODO: Eventually we want to not have this hardcoded, but pyuvdata at
+            # present does not carry around any velocity information. As per usual,
+            # I (Karto) am tipping my hand on what I might be working on next...
+            su_hdu.header["VELTYP"] = "LSR"
+            fits_tables.append(su_hdu)
+
         # write the file
         hdulist = fits.HDUList(hdus=fits_tables)
         hdulist.writeto(filename, overwrite=True)
